{
    "rm_token1": "UPDATE ir_config_parameter SET value='***' WHERE key='module_example1.access_token'",
    "rm_token2": "UPDATE ir_config_parameter SET value='***' WHERE key='module_example1.access_token'"
}
